# coding: utf-8

import base64
import hashlib
import json
import urllib
import urllib2

import re

from acomea.sso.utils.sso_client import SSOClient
from django import forms
from django.conf import settings
from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import User
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.utils.html import mark_safe

from captcha.fields import ReCaptchaField

from acomea.tracerequest.models import TraceRequestData, Gimme5RegistrationData, FaiComeIlFaiRegistrationData, Gimme5CampaignRegistrationData
from acomea.contact.models import Contact
from acomea.tracerequest.cms.models import TraceRequestSettings, Gimme5RegistrationFormSettings, FaiComeIlFaiRegistrationFormSettings, Gimme5CampaignRegistrationFormSettings
from jmb.core.middleware.force_response import ForceResponse


class TraceRequestPluginForm(forms.ModelForm):
    class Meta:
        model = TraceRequestSettings

    def __init__(self, *args, **kwargs):
        super(TraceRequestPluginForm, self).__init__(*args, **kwargs)
        self.fields['activation_thanks_url'].required = True
        self.fields['request_email'].required = True


class EventTraceRequestPluginForm(forms.ModelForm):
    class Meta:
        model = TraceRequestSettings

    def __init__(self, *args, **kwargs):
        super(EventTraceRequestPluginForm, self).__init__(*args, **kwargs)
        self.fields['event'].required = True


class TraceRequestForm(forms.ModelForm):
    username = forms.RegexField(
        label=_("Username"), max_length=50,
        regex=r'^[\w.+-]+$',
        help_text=_(
            "Required. 50 characters or fewer. Letters, digits and "
            "./+/-/_ only."
        ),
        error_messages={
            'invalid': _(
                "This value may contain only letters, numbers and "
                "./+/-/_ characters."
            )
        }
    )

    class Meta:
        model = TraceRequestData
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier', 'plugin_setting')

    def __init__(self, *args, **kwargs):
        self.plugin_instance = kwargs.pop('plugin_instance')
        self.uid = kwargs.pop('uid')
        super(TraceRequestForm, self).__init__(*args, **kwargs)
        self.prepare_fields()

    def prepare_fields(self):
        # Prepare form fields in case of registration and trace request
        del self.fields['status']
        self.fields['phone'].required = self.plugin_instance.phone_required
        self.fields['message'].required = self.plugin_instance.message_required
        if self.plugin_instance.has_captcha:
            self.fields['captcha'] = ReCaptchaField(label='')
        if self.plugin_instance.has_registration:
            self.fields['first_name'].required = True
            self.fields['last_name'].required = True
            self.fields['username'].required = True
            self.fields['password'].required = True
            self.fields['password'].widget = forms.PasswordInput()
            self.fields['password_confirm'] = forms.CharField(
                widget=forms.PasswordInput(),
                required=True, label=_('Password confirm'),
            )
            self.fields['professionale'] = forms.BooleanField(
                required=False
            )
        # Prepare form fields in case of simple trace request
        else:
            self.fields['first_name'].required = self.plugin_instance.first_name_required
            self.fields['last_name'].required = self.plugin_instance.last_name_required
            del self.fields['username']
            del self.fields['password']
            del self.fields['professionale']
            if not self.plugin_instance.first_name_present:
                del self.fields['first_name']
            if not self.plugin_instance.last_name_present:
                del self.fields['last_name']
        # Phone and message could be not present in both cases
        if not self.plugin_instance.phone_present:
            del self.fields['phone']
        if not self.plugin_instance.message_present:
            del self.fields['message']
        # Auth trattamento dati
        if self.plugin_instance.has_auth_trattamento_dati:
            self.fields['auth_trattamento_dati'].required = self.plugin_instance.requires_auth_trattamento_dati
            self.fields['auth_trattamento_dati'].widget = forms.RadioSelect(
                choices=settings.TRACEREQUEST_AUTH_CHOICES,
            )
            self.fields['auth_trattamento_dati'].label = _("Autorizzazione trattamento dati")
            # self.fields['auth_trattamento_dati'].initial = 0
        else:
            del self.fields['auth_trattamento_dati']
        # Auth materiale commerciale
        if self.plugin_instance.has_auth_materiale_commerciale:
            self.fields['auth_materiale_commerciale'].required = self.plugin_instance.requires_auth_materiale_commerciale
            self.fields['auth_materiale_commerciale'].widget = forms.RadioSelect(
                choices=settings.TRACEREQUEST_AUTH_CHOICES
            )
            self.fields['auth_materiale_commerciale'].label = _(
                "Autorizzazione materiale commerciale"
            )
            # self.fields['auth_materiale_commerciale'].initial = 0
        else:
            del self.fields['auth_materiale_commerciale']
        # Auth societa terze
        if self.plugin_instance.has_auth_societa_terze:
            self.fields['auth_societa_terze'].required = self.plugin_instance.requires_auth_societa_terze
            self.fields['auth_societa_terze'].widget = forms.RadioSelect(
                choices=settings.TRACEREQUEST_AUTH_CHOICES
            )
            self.fields['auth_societa_terze'].label = _("Autorizzazione societa terze")
            # self.fields['auth_societa_terze'].initial = 0
        else:
            del self.fields['auth_societa_terze']

        if self.uid:
            try:
                email = base64.b64decode(self.uid)
                contact = Contact.objects.get(emails__email__iexact=email)
                self.fields['email'].initial = email
                self.fields['email'].widget.attrs['readonly'] = True
                if "first_name" in self.fields:
                    self.fields['first_name'].initial = contact.first_name
                    if contact.user:
                        self.fields['first_name'].widget.attrs['readonly'] = True
                if "last_name" in self.fields:
                    self.fields['last_name'].initial = contact.last_name
                    if contact.user:
                        self.fields['last_name'].widget.attrs['readonly'] = True
            except Exception:
                pass

    def save(self, commit=True, *args, **kwargs):
        plugin = self.plugin_instance
        trace_request = super(TraceRequestForm, self).save(commit=False, *args, **kwargs)
        if 'password' in self.cleaned_data:
            trace_request.raw_password = self.cleaned_data['password']
            trace_request.password = make_password(self.cleaned_data['password'])
        if not plugin.has_auth_trattamento_dati:
            trace_request.auth_trattamento_dati = True
        trace_request.plugin_setting = plugin
        trace_request.save()
        return trace_request

    def clean(self):
        data = self.cleaned_data
        if self.plugin_instance.password_present and 'password' in data and 'password_confirm' in data:
            if data['password'] != data['password_confirm']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return data

    def clean_email(self):
        from acomea.ldap import ldap_models
        data = self.cleaned_data
        email = data['email'].lower()
        error_message = mark_safe(
            "Un utente con questa email esiste già. "
            "<a href='/gestione-password/recupera-password/'>recupera password</a>"
        )
        # We need to clean email field only if plugin is used for registration
        if self.plugin_instance.has_registration:
            try:
                user = User.objects.get(email__iexact=email)
            except User.MultipleObjectsReturned:
                raise forms.ValidationError(error_message)
            except User.DoesNotExist:
                if TraceRequestData.objects.filter(email__iexact=email).count():
                    raise forms.ValidationError(error_message)

                if settings.LDAP_SERVER:
                    ldap_user = ldap_models.get_user_by_email(email)
                    if not ldap_user:
                        return email
                    raise forms.ValidationError(error_message)

                if settings.ACOMEA_SSO_SERVER:
                    # verifica email solo se abilitato ACOMEA_SSO_SERVER
                    response = SSOClient().exist(email=email, get_response=True)
                    if response.status_code == 404:
                        return email
                    if response.status_code == 200:
                        raise forms.ValidationError(error_message)
                    raise forms.ValidationError(response.reason)

                return email
            raise forms.ValidationError(error_message)
        return email

    def clean_username(self):
        from acomea.ldap import ldap_models
        username = self.cleaned_data.get('username').lower()
        error_message = mark_safe(
            "Un utente con questo nickname esiste già. "
            "<a href='/gestione-password/recupera-password/'>recupera password</a>"
        )
        # We need to clean email field only if plugin is used for registration
        if self.plugin_instance.has_registration:
            try:
                user = User.objects.get(username=username)
            except User.DoesNotExist:
                if TraceRequestData.objects.filter(username=username).count():
                    raise forms.ValidationError(error_message)

                if settings.LDAP_SERVER:
                    ldap_user = ldap_models.get_user_by_uid(username)
                    if not ldap_user:
                        return username
                    raise forms.ValidationError(error_message)

                if settings.ACOMEA_SSO_SERVER:
                    # verifica username solo se abilitato ACOMEA_SSO_SERVER
                    response = SSOClient().exist(username=username, get_response=True)
                    if response.status_code == 404:
                        return username
                    if response.status_code == 200:
                        raise forms.ValidationError(error_message)
                    raise forms.ValidationError(response.reason)

                return username

            except User.MultipleObjectsReturned:
                raise forms.ValidationError(error_message)
            raise forms.ValidationError(error_message)
        return username

    def clean_password(self):
        password = self.cleaned_data.get('password')
        # We need to clean email field only if plugin is used for registration
        if self.plugin_instance.has_registration:
            if len(password) < 8:
                raise forms.ValidationError(
                    "La password deve essere di almeno 8 caratteri."
                )
            first_isalpha = password[0].isalpha()
            if all(c.isalpha() == first_isalpha for c in password):
                raise forms.ValidationError(
                    "La password deve contenere almeno una lettera e un numero "
                    "o carattere di punteggiatura."
                )
            try:
                password.decode('ascii')
            except Exception, e:
                raise forms.ValidationError(
                    "La password deve contenere solamente caratteri ascii "
                    "(-a-zA-Z0-1._$&*(){}\[\]+=<>?\|~@#/)"
                )
        return password

    def clean_password_confirm(self):
        password_confirm = self.cleaned_data.get('password_confirm')
        # We need to clean email field only if plugin is used for registration
        if self.plugin_instance.has_registration:
            if len(password_confirm) < 8:
                raise forms.ValidationError(
                    "La password deve essere di almeno 8 caratteri."
                )
            first_isalpha = password_confirm[0].isalpha()
            if all(c.isalpha() == first_isalpha for c in password_confirm):
                raise forms.ValidationError(
                    "La password deve contenere almeno una lettera e un numero "
                    "o carattere di punteggiatura."
                )
            try:
                password_confirm.decode('ascii')
            except Exception:
                raise forms.ValidationError(
                    "La password deve contenere solamente caratteri ascii "
                    "(-a-zA-Z0-1._$&*(){}\[\]+=<>?\|~@#/)"
                )
        return password_confirm

    def clean_phone(self):
        phone = self.cleaned_data['phone']
        # min length 7 digit AND only digit
        if phone != '':
            # se phone solo digit --> len(phone) >=7
            # se phone comincia per + --> len(phone) >=8; i prefissi vanno da +1 a +998
            # solo digit e il + all'inizio sono permessi
            # Rimuovo anche gli spazi intemedi

            pattern = re.compile(r"((^[+])?(\d{1,3}))?[0-9]{7,}$")
            phone = phone.replace(' ', '')
            match = pattern.match(phone)
            if not match:
                raise forms.ValidationError(_('invalid number'))

        return phone

    def clean_auth_trattamento_dati(self):
        auth = self.cleaned_data['auth_trattamento_dati']
        if self.plugin_instance.requires_auth_trattamento_dati and auth == 0:
            raise forms.ValidationError(settings.AUTH_TRATTAMENTO_DATI_ERR_MSG)
        return auth

    def clean_auth_business_privacy(self):
        auth = self.cleaned_data['auth_business_privacy']
        if self.plugin_instance.requires_auth_business_privacy and auth == 0:
            raise forms.ValidationError(settings.AUTH_MATERIALE_COMMERCIALE_ERR_MSG)
        return auth

    def clean_auth_societa_terze(self):
        auth = self.cleaned_data['auth_societa_terze']
        if self.plugin_instance.requires_auth_societa_terze and auth == 0:
            raise forms.ValidationError(settings.AUTH_SOCIETA_TERZE_ERR_MSG)
        return auth


class Gimme5RegistrationPluginForm(forms.ModelForm):
    class Meta:
        model = Gimme5RegistrationFormSettings

    def __init__(self, *args, **kwargs):
        super(Gimme5RegistrationPluginForm, self).__init__(*args, **kwargs)
        self.fields['activation_thanks_url'].required = True


class Gimme5RegistrationForm(forms.ModelForm):

    class Meta:
        model = Gimme5RegistrationData
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier', 'plugin_setting')

    def __init__(self, *args, **kwargs):
        try:
            self.plugin_instance = kwargs.pop('plugin_instance')
            super(Gimme5RegistrationForm, self).__init__(*args, **kwargs)
            self.prepare_fields()
        except KeyError:
            super(Gimme5RegistrationForm, self).__init__(*args, **kwargs)

    def prepare_fields(self):
        # Prepare form fields both in case of login and registration
        self.fields['email'].required = True
        self.fields['email'].widget.attrs['placeholder'] = 'email o nome utente'
        self.fields['password'].required = True
        self.fields['password'].widget = forms.PasswordInput(attrs={'placeholder': 'password'})
        # Prepare form fields in case of registration
        if self.plugin_instance.has_registration:
            self.fields['email'].widget.attrs['placeholder'] = 'email'
            self.fields['first_name'].required = True
            self.fields['first_name'].widget.attrs['placeholder'] = 'nome'
            self.fields['last_name'].required = True
            self.fields['last_name'].widget.attrs['placeholder'] = 'cognome'
            self.fields['password_confirm'] = forms.CharField(
                widget=forms.PasswordInput(),
                required=True, label=_('Password confirm')
            )
            self.fields['password_confirm'].widget.attrs['placeholder'] = 'Password confirm'
        else:
            del self.fields['first_name']
            del self.fields['last_name']
            del self.fields['password_confirm']
        # Auth trattamento dati
        if self.plugin_instance.has_auth_trattamento_dati:
            self.fields['auth_trattamento_dati'].required = self.plugin_instance.requires_auth_trattamento_dati
            self.fields['auth_trattamento_dati'].widget = forms.RadioSelect(
                choices=settings.TRACEREQUEST_AUTH_CHOICES,
            )
            self.fields['auth_trattamento_dati'].label = _("Autorizzazione trattamento dati")
        else:
            del self.fields['auth_trattamento_dati']
        # Captcha (Captcha doesn't need to be deleted if not present cause it's not in Gimme5RegistrationData model)
        if self.plugin_instance.has_captcha:
            self.fields['captcha'] = ReCaptchaField(label='')


# Acutally Gimme5 Campaign generic Registration Plugin Form
class FaiComeIlFaiRegistrationPluginForm(forms.ModelForm):
    class Meta:
        model = FaiComeIlFaiRegistrationFormSettings

    def __init__(self, *args, **kwargs):
        super(FaiComeIlFaiRegistrationPluginForm, self).__init__(*args, **kwargs)


# Acutally Gimme5 Campaign generic Registration Form
class FaiComeIlFaiRegistrationForm(forms.ModelForm):

    class Meta:
        model = FaiComeIlFaiRegistrationData
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier', 'plugin_setting')

    def __init__(self, *args, **kwargs):
        try:
            self.plugin_instance = kwargs.pop('plugin_instance')
            super(FaiComeIlFaiRegistrationForm, self).__init__(*args, **kwargs)
            self.prepare_fields()
        except KeyError:
            super(FaiComeIlFaiRegistrationForm, self).__init__(*args, **kwargs)

    def prepare_fields(self):
        # Prepare form fields both in case of login and registration
        self.fields['email'].required = True
        self.fields['email'].widget.attrs['placeholder'] = 'email'
        self.fields['password'].required = True
        self.fields['password'].widget = forms.PasswordInput(attrs={'placeholder': 'password'})
        self.fields['first_name'].required = True
        self.fields['first_name'].widget.attrs['placeholder'] = 'nome'
        self.fields['last_name'].required = True
        self.fields['last_name'].widget.attrs['placeholder'] = 'cognome'
        self.fields['fiscal_code'].required = True
        self.fields['fiscal_code'].widget.attrs['placeholder'] = 'codice fiscale'
        self.fields['password_confirm'] = forms.CharField(
            widget=forms.PasswordInput(),
            required=True, label=_('Password confirm')
        )
        self.fields['password_confirm'].widget.attrs['placeholder'] = 'Password confirm'
        self.fields['auth_privacy'].required = True
        self.fields['auth_privacy'].widget = forms.RadioSelect(
            choices=settings.TRACEREQUEST_AUTH_CHOICES,
        )
        self.fields['auth_privacy'].label = _("Accetta l'informativa sulla privacy")

    def clean(self):
        data = self.cleaned_data
        if self.plugin_instance.password_present and 'password' in data and 'password_confirm' in data:
            if data['password'] != data['password_confirm']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return data


# Acutally Gimme5 Campaign generic Registration Plugin Form
class Gimme5CampaignRegistrationPluginForm(forms.ModelForm):
    class Meta:
        model = Gimme5CampaignRegistrationFormSettings

    def __init__(self, *args, **kwargs):
        super(Gimme5CampaignRegistrationPluginForm, self).__init__(*args, **kwargs)


# Acutally Gimme5 Campaign generic Registration Form
class Gimme5CampaignRegistrationForm(forms.ModelForm):

    class Meta:
        model = Gimme5CampaignRegistrationData
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier', 'plugin_setting')

    def __init__(self, *args, **kwargs):
        try:
            self.plugin_instance = kwargs.pop('plugin_instance')
            super(Gimme5CampaignRegistrationForm, self).__init__(*args, **kwargs)
            self.prepare_fields()
        except KeyError:
            super(Gimme5CampaignRegistrationForm, self).__init__(*args, **kwargs)

    def prepare_fields(self):
        # Prepare form fields both in case of login and registration
        self.fields['email'].required = True
        self.fields['email'].widget.attrs['placeholder'] = 'email'
        self.fields['password'].required = True
        self.fields['password'].widget = forms.PasswordInput(attrs={'placeholder': 'password'})
        self.fields['first_name'].required = True
        self.fields['first_name'].widget.attrs['placeholder'] = 'nome'
        self.fields['last_name'].required = True
        self.fields['last_name'].widget.attrs['placeholder'] = 'cognome'
        self.fields['fiscal_code'].required = True
        self.fields['fiscal_code'].widget.attrs['placeholder'] = 'codice fiscale'
        self.fields['password_confirm'] = forms.CharField(
            widget=forms.PasswordInput(),
            required=True, label=_('Password confirm')
        )
        self.fields['password_confirm'].widget.attrs['placeholder'] = 'Password confirm'
        self.fields['auth_privacy'].required = True
        self.fields['auth_privacy'].widget = forms.RadioSelect(
            choices=settings.TRACEREQUEST_AUTH_CHOICES,
        )
        self.fields['auth_privacy'].label = _("Accetta l'informativa sulla privacy")

    def clean(self):
        data = self.cleaned_data
        if self.plugin_instance.password_present and 'password' in data and 'password_confirm' in data:
            if data['password'] != data['password_confirm']:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return data