# coding: utf-8
import base64
import cookielib
import datetime
import logging
import json
import urllib
import urllib2
import hashlib


from allauth.account.signals import user_signed_up
from allauth.account.utils import perform_login
from allauth.socialaccount.signals import pre_social_login
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.db import models
from django.dispatch import receiver
from django.http import HttpResponseRedirect
from django.utils.http import int_to_base36
from django.utils.translation import ugettext_lazy as _
from django.template import Context, Template
from django.utils.safestring import mark_safe

from jmb.core.db.fields import HTMLFieldTinyMCE
from jmb.core.db.models import DateModel, UserModel, StatusModel
from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.functions import zerofill
from jmb.core.utils.token import TokenGenerator

from urlparse import urlparse, parse_qs

from acomea.ldap import ldap_models
from acomea.sso.utils import sso_client
from acomea.tracerequest.utils import save_as_registration_plugin, save_as_tracerequest_plugin


class TraceRequestData(DateModel, UserModel, StatusModel):
    """
    Model used by CMS to record trace request data
    """
    email = models.EmailField(
        verbose_name=_('email'),
    )
    first_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('first name'),
    )
    last_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('last name'),
    )
    phone = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_('phone'),
    )
    message = models.TextField(
        blank=True, null=True,
        verbose_name=_('message'),
    )
    username = models.CharField(
        max_length=50,
        blank=True, null=True,
        verbose_name=_('username'),
    )
    raw_password = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('raw password'),
    )
    password = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password'),
    )
    auth_trattamento_dati = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth trattamento dati'),
    )
    auth_materiale_commerciale = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth materiale commerciale'),
    )
    auth_societa_terze = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth societa terze'),
    )
    plugin_setting = models.ForeignKey(
        "TraceRequestSettings",
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='trace_request_data',
        verbose_name=_('plugin settings'),
    )
    iscrizione_alla_newsletter = models.BooleanField(
        verbose_name=_("Iscrizione alla newsletter"),
        default=True
    )
    professionale = models.BooleanField(
        verbose_name=_("Professionale"),
        default=False,
    )
    utm_campaign = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('campaign'),
    )
    utm_source = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('source'),
    )

    class Meta:
        verbose_name = _('trace request data')
        verbose_name_plural = _('trace requests data')

    def __unicode__(self):
        return u'{0}'.format(self.email)

    def save(self, *args, **kwargs):
        if self.first_name is None or (isinstance(self.first_name, str) and self.first_name.strip() == ""):
            self.first_name = self.email.split('@')[0]
        self.last_name = self.last_name or ""
        super(TraceRequestData, self).save(*args, **kwargs)

    def send_information_request_email(self):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = [setting.email_receiver]
        subject = setting.information_request_email.subject
        # Create context and renders email content
        html_message = Template(setting.information_request_email.body)
        context_dictionary = self.__dict__
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_post_email(self, path):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = [setting.post_email_receiver]
        subject = 'Compilazione traccia richieste'
        # Create context and renders email content
        body = "L'utente <strong>{}</strong> ha compilato il traccia richieste alla pagina {}".format(
            self.email, path
        )
        html_message = Template(body)
        context_dictionary = dict()
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def get_confirmation_url(self):
        return "{0}://{1}{2}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            reverse('tracerequest:trace_request_post_registration', args=[self.uidb36, self.token]),
        )
    get_confirmation_url.short_description = _('confirmation url')

    def send_post_registration_email(self, alternative_email=None):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = (self.email,)
        if alternative_email:
            to = (alternative_email,)

        subject = setting.request_email.subject
        # Create context and renders email content
        html_message = Template(setting.request_email.body)
        context_dictionary = self.__dict__
        # Confirm management
        # confirm_url = "{0}://{1}{2}".format(
        #     settings.SITE_PROTOCOL,
        #     settings.SITE_DOMAIN,
        #     reverse('tracerequest:trace_request_post_registration', args=[self.uidb36, self.token]),
        # )
        confirm_url = self.get_confirmation_url()

        context_dictionary['confirm_link'] = mark_safe(u"<a href='%s'>qui</a>" % confirm_url)
        # Registration management
        registration_url = "{0}://{1}{2}?uid={3}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            setting.contact_registration_url,
            base64.b64encode(self.email)
        )
        context_dictionary['registration_link'] = mark_safe(u"<a href='%s'>qui</a>" % registration_url)
        # Packing context
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Sends email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_post_activation_email(self):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = (self.email,)
        subject = setting.confirm_email.subject
        # Create context and renders email content
        html_message = Template(setting.confirm_email.body)
        context_dictionary = self.__dict__
        # Registration management
        registration_url = "{0}://{1}{2}?uid={3}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            setting.contact_registration_url,
            base64.b64encode(self.email)
        )
        context_dictionary['registration_link'] = mark_safe(u"<a href='%s'>qui</a>" % registration_url)
        # Packing context
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def id_to_base_36(self):
        return int_to_base36(self.id)
    uidb36 = property(id_to_base_36)

    def get_code_for_token(self):
        """
        Returns code used for token
        """
        return u"%s-%s" % (
            zerofill(self.id, 10),
            self.date_last_modify.replace(tzinfo=None)
        )
    code_for_token = property(get_code_for_token)

    def get_token(self):
        """
        Returns token string of Buyer
        """
        token_generator = TokenGenerator()
        return token_generator.make_token(self.code_for_token)
    token = property(get_token)

    def check_token(self, token, duration=None):
        """
        Verify the validity of Buyer
        """
        token_generator = TokenGenerator()
        if not token_generator.check_token(self.code_for_token, token):
            return False
        if duration and datetime.datetime.now() > self.date_create + datetime.timedelta(seconds=duration):
            return False
        return True


class Gimme5RegistrationData(DateModel, UserModel):
    """
    Model used by CMS to populate Form
    """
    email = models.EmailField(
        verbose_name=_('email'),
    )
    password_confirm = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password confirm'),
    )
    first_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('first name'),
    )
    last_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('last name'),
    )
    password = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password'),
    )
    auth_trattamento_dati = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth trattamento dati'),
    )
    plugin_setting = models.ForeignKey(
        "Gimme5RegistrationFormSettings",
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='gimme5_registration_data',
        verbose_name=_('plugin settings'),
    )

    class Meta:
        verbose_name = _('gimme5 registration data')
        verbose_name_plural = _('gimme5 registration data')

    def __unicode__(self):
        return u'{0}'.format(self.email)

    def send_information_request_email(self):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = [setting.email_receiver]
        subject = setting.information_request_email.subject
        # Create context and renders email content
        html_message = Template(setting.information_request_email.body)
        context_dictionary = self.__dict__
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_post_email(self, path):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = [setting.post_email_receiver]
        subject = 'Compilazione traccia richieste'
        # Create context and renders email content
        body = "L'utente <strong>{}</strong> ha compilato il traccia richieste alla pagina {}".format(
            self.email, path
        )
        html_message = Template(body)
        context_dictionary = dict()
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_post_registration_email(self, alternative_email=None):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = (self.email,)
        if alternative_email:
            to = (alternative_email,)

        subject = setting.request_email.subject
        # Create context and renders email content
        html_message = Template(setting.request_email.body)
        context_dictionary = self.__dict__
        # Confirm management
        confirm_url = "{0}://{1}{2}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            reverse('tracerequest:trace_request_post_registration', args=[self.uidb36, self.token]),
        )
        context_dictionary['confirm_link'] = mark_safe(u"<a href='%s'>qui</a>" % confirm_url)
        # Registration management
        registration_url = "{0}://{1}{2}?uid={3}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            setting.contact_registration_url,
            base64.b64encode(self.email)
        )
        context_dictionary['registration_link'] = mark_safe(u"<a href='%s'>qui</a>" % registration_url)
        # Packing context
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Sends email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def send_post_activation_email(self):
        setting = self.plugin_setting
        email_sender = setting.email_sender
        to = (self.email,)
        subject = setting.confirm_email.subject
        # Create context and renders email content
        html_message = Template(setting.confirm_email.body)
        context_dictionary = self.__dict__
        # Registration management
        registration_url = "{0}://{1}{2}?uid={3}".format(
            settings.SITE_PROTOCOL,
            settings.SITE_DOMAIN,
            setting.contact_registration_url,
            base64.b64encode(self.email)
        )
        context_dictionary['registration_link'] = mark_safe(u"<a href='%s'>qui</a>" % registration_url)
        # Packing context
        context = Context(context_dictionary)
        html_message = html_message.render(context)
        # Email subject *must not* contain newlines
        subject = ''.join(subject.splitlines())
        text_message = html_message
        # Send email
        msg = EmailMultiAlternatives(subject=subject, body=text_message, from_email=email_sender, to=to)
        msg.attach_alternative(html_message, "text/html")
        msg.send()

    def id_to_base_36(self):
        return int_to_base36(self.id)
    uidb36 = property(id_to_base_36)

    def get_code_for_token(self):
        """
        Returns code used for token
        """
        return u"%s-%s" % (
            zerofill(self.id, 10),
            self.date_last_modify.replace(tzinfo=None)
        )
    code_for_token = property(get_code_for_token)

    def get_token(self):
        """
        Returns token string of Buyer
        """
        token_generator = TokenGenerator()
        return token_generator.make_token(self.code_for_token)
    token = property(get_token)

    def check_token(self, token, duration=None):
        """
        Verify the validity of Buyer
        """
        token_generator = TokenGenerator()
        if not token_generator.check_token(self.code_for_token, token):
            return False
        if duration and datetime.datetime.now() > self.date_create + datetime.timedelta(seconds=duration):
            return False
        return True


class FaiComeIlFaiRegistrationData(DateModel, UserModel):
    """
    Model used by CMS to populate Form
    """
    email = models.EmailField(
        verbose_name=_('email'),
    )
    first_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('first name'),
    )
    last_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('last name'),
    )
    password = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password'),
    )
    password_confirm = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password confirm'),
    )
    auth_privacy = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth privacy'),
    )
    fiscal_code = models.CharField(
        max_length=16,
        blank=True, null=True,
        verbose_name=_('fiscal code'),
    )
    plugin_setting = models.ForeignKey(
        "FaiComeIlFaiRegistrationFormSettings",
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='faicomeilfai_registration_data',
        verbose_name=_('plugin settings'),
    )

    class Meta:
        verbose_name = _('Fai come il Fai registration data')
        verbose_name_plural = _('Fai come il Fai registration data')

    def __unicode__(self):
        return u'{0}'.format(self.email)


class Gimme5CampaignRegistrationData(DateModel, UserModel):
    """
    Model used by CMS to populate Form
    """
    email = models.EmailField(
        verbose_name=_('email'),
    )
    first_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('first name'),
    )
    last_name = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('last name'),
    )
    password = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password'),
    )
    password_confirm = models.CharField(
        max_length=128,
        blank=True, null=True,
        verbose_name=_('password confirm'),
    )
    auth_privacy = models.IntegerField(
        blank=True, null=True,
        choices=settings.TRACEREQUEST_AUTH_CHOICES,
        verbose_name=_('auth privacy'),
    )
    fiscal_code = models.CharField(
        max_length=16,
        blank=True, null=True,
        verbose_name=_('fiscal code'),
    )
    plugin_setting = models.ForeignKey(
        "Gimme5CampaignRegistrationFormSettings",
        blank=True, null=True,
        on_delete=models.SET_NULL,
        related_name='gimme5campaign_registration_data',
        verbose_name=_('plugin settings'),
    )

    class Meta:
        verbose_name = _('Gimme5 campaign registration data')
        verbose_name_plural = _('Gimme5 campaign Fai registration data')

    def __unicode__(self):
        return u'{0}'.format(self.email)


class EmailMessage(DateModel, UserModel):
    """
    Model used by CMS for create email message in tracerequest.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    subject = models.CharField(
        max_length=255,
        verbose_name=_('subject'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('email message'),
    )

    class Meta:
        verbose_name = _('email message')
        verbose_name_plural = _('email messages')

    def __unicode__(self):
        return u"{0}".format(self.name)


class PrivacyMessage(DateModel, UserModel):
    """
    Model used by CMS for create privacy message in tracerequest.
    """
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    body = HTMLFieldTinyMCE(
        verbose_name=_('privacy message'),
    )

    class Meta:
        verbose_name = _('privacy message')
        verbose_name_plural = _('privacy messages')

    def __unicode__(self):
        return u"{0}".format(self.name)


################################################################
# Signals ######################################################
################################################################
@receiver(pre_social_login)
def connect_account(sender, request, sociallogin, **kwargs):
    logger = logging.getLogger('acomea.social.warning')

    user = sociallogin.user

    email = user.email if hasattr(user, 'email') else ''
    account = user.account if hasattr(user, 'account') else ''
    username = user.username if hasattr(user, 'username') else ''

    log_message = 'username:{}\temail:{}\taccount:{}'.format(username, email, account)

    if not user.email:
        i = 1
        logger.warning('{} {}'.format(i, log_message))
        raise User.DoesNotExist

    if settings.LDAP_SERVER:
        ldap_user = ldap_models.get_user_by_email(user.email)
        if ldap_user:
            ldap_user.set_otp()
            ldap_user.save()
            i = 2
            logger.warning('{} {}'.format(i, log_message))

    if settings.ACOMEA_SSO_SERVER:
        try:
            login_data = sso_client.SSOClient().social_token(username=user.username)
            setattr(user, 'sso', login_data)
        except Exception:
            logger.error(log_message)

    if user.id:
        i = 3
        logger.warning('{} {}'.format(i, log_message))
        return
    try:
        user = User.objects.get(email=user.email)
        sociallogin.state['process'] = 'connect'
        perform_login(request, user, 'none')
    except User.DoesNotExist as e:
        log_message = '{}\n{}'.format(log_message, e)
        logger.warning(log_message)
    except User.MultipleObjectsReturned as e:
        log_message = '{}\n{}'.format(log_message, e)
        logger.warning(log_message)
    except Exception:
        logger.error(log_message)


@receiver(user_signed_up)
def social_post_sign_up(sender, request, sociallogin, **kwargs):
    logger = logging.getLogger('acomea.social.warning')

    user = sociallogin.user
    data = dict(
        email=user.email,
        first_name=user.first_name,
        last_name=user.last_name,
        auth_trattamento_dati=True,
        auth_materiale_commerciale=None,
    )

    email = user.email if hasattr(user, 'email') else ''
    account = user.account if hasattr(user, 'account') else ''
    username = user.username if hasattr(user, 'username') else ''
    log_message = 'username:{}\temail:{}\taccount:{}'.format(username, email, account)

    if not user.email:
        i = 11
        logger.warning('{} {}'.format(i, log_message))
        raise User.DoesNotExist

    save_as_registration_plugin(data, user)

    utm_campaign = request.COOKIES.get('utm_campaign', None)
    utm_source = request.COOKIES.get('utm_source', None)
    if utm_campaign or utm_source:
        data.update(
            {
                'utm_campaign': utm_campaign,
                'utm_source': utm_source,
            }
        )
    save_as_tracerequest_plugin(data)
    if settings.LDAP_SERVER:
        ldap_user = ldap_models.get_user_by_email(user.email)
        if ldap_user:
            i = 12
            logger.warning('{} {}'.format(i, log_message))
            ldap_user.set_otp()
            ldap_user.save()

    if settings.ACOMEA_SSO_SERVER:
        try:
            login_data = sso_client.SSOClient().social_token(username=user.username)
            setattr(user, 'sso', login_data)
        except Exception:
            logger.error(log_message)


if settings.CMS_ENABLED:
    import acomea.tracerequest.cms.plugins
    import acomea.tracerequest.cms.app
